/*
 * This Software (C) Copyright David Goodwin, 2008, 2009.
 * 
 *   This is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This software is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this software; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package iasel.gui;

import iasel.*;
import iasel.data.AreaProfile;
import iasel.data.REntry;
import iasel.interfaces.StatusMonitor;
import iasel.gui.ImageCalcPanel;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.io.File;
import java.util.ArrayList;
import javax.imageio.ImageIO;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.filechooser.FileNameExtensionFilter;

/**
 * The application's main frame.
 */
public class IaselView extends FrameView implements StatusMonitor{

    BufferedImage bImage;
    int status = 0;
    ResultsWindow rw;
    WorkQueue wq;
    File currentFile = null;
    boolean currentDone = false;
    boolean fromQueue = false;
    
    
    public IaselView(SingleFrameApplication app) {
        super(app);

        initComponents();

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
        
        
        // ---------------------------------
        imageCalcPanel1.setMonitor(this);
        wq = new WorkQueue(mnuViewWorkQueue);
        rw = new ResultsWindow(mnuViewResultsWindow);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        imageCalcPanel1 = new iasel.gui.ImageCalcPanel();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        mnuFileOpen = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        mnuFileAddFile = new javax.swing.JMenuItem();
        mnuFileAddDirectory = new javax.swing.JMenuItem();
        jSeparator7 = new javax.swing.JSeparator();
        mnuFileExportCSV = new javax.swing.JMenuItem();
        jSeparator8 = new javax.swing.JSeparator();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        jMenu1 = new javax.swing.JMenu();
        mnuViewWorkQueue = new javax.swing.JCheckBoxMenuItem();
        mnuViewResultsWindow = new javax.swing.JCheckBoxMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();
        jToolBar1 = new javax.swing.JToolBar();
        jToggleButton3 = new javax.swing.JToggleButton();
        jToggleButton4 = new javax.swing.JToggleButton();
        jToggleButton1 = new javax.swing.JToggleButton();
        jToggleButton2 = new javax.swing.JToggleButton();
        jSeparator2 = new javax.swing.JToolBar.Separator();
        jButton1 = new javax.swing.JButton();
        jSeparator3 = new javax.swing.JToolBar.Separator();
        jButton4 = new javax.swing.JButton();
        jButton5 = new javax.swing.JButton();
        buttonGroup1 = new javax.swing.ButtonGroup();
        jSeparator4 = new javax.swing.JSeparator();
        jSeparator5 = new javax.swing.JSeparator();
        jSeparator6 = new javax.swing.JSeparator();

        mainPanel.setName("mainPanel"); // NOI18N
        mainPanel.setLayout(new java.awt.GridLayout(1, 0));

        imageCalcPanel1.setName("imageCalcPanel1"); // NOI18N
        mainPanel.add(imageCalcPanel1);

        menuBar.setName("menuBar"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(iasel.Iasel.class).getContext().getResourceMap(IaselView.class);
        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(iasel.Iasel.class).getContext().getActionMap(IaselView.class, this);
        mnuFileOpen.setAction(actionMap.get("fileOpenImage")); // NOI18N
        mnuFileOpen.setIcon(resourceMap.getIcon("mnuOpenImage.icon")); // NOI18N
        mnuFileOpen.setText(resourceMap.getString("mnuOpenImage.text")); // NOI18N
        mnuFileOpen.setName("mnuOpenImage"); // NOI18N
        fileMenu.add(mnuFileOpen);

        jSeparator1.setName("jSeparator1"); // NOI18N
        fileMenu.add(jSeparator1);

        mnuFileAddFile.setAction(actionMap.get("addFileToQueue")); // NOI18N
        mnuFileAddFile.setIcon(resourceMap.getIcon("mnuFileAddFile.icon")); // NOI18N
        mnuFileAddFile.setText(resourceMap.getString("mnuFileAddFile.text")); // NOI18N
        mnuFileAddFile.setName("mnuFileAddFile"); // NOI18N
        fileMenu.add(mnuFileAddFile);

        mnuFileAddDirectory.setAction(actionMap.get("addDirectoryToQueue")); // NOI18N
        mnuFileAddDirectory.setIcon(resourceMap.getIcon("mnuFileAddDirectory.icon")); // NOI18N
        mnuFileAddDirectory.setText(resourceMap.getString("mnuFileAddDirectory.text")); // NOI18N
        mnuFileAddDirectory.setName("mnuFileAddDirectory"); // NOI18N
        fileMenu.add(mnuFileAddDirectory);

        jSeparator7.setName("jSeparator7"); // NOI18N
        fileMenu.add(jSeparator7);

        mnuFileExportCSV.setAction(actionMap.get("exportToCSV")); // NOI18N
        mnuFileExportCSV.setIcon(resourceMap.getIcon("mnuFileExportCSV.icon")); // NOI18N
        mnuFileExportCSV.setName("mnuFileExportCSV"); // NOI18N
        fileMenu.add(mnuFileExportCSV);

        jSeparator8.setName("jSeparator8"); // NOI18N
        fileMenu.add(jSeparator8);

        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/iasel/gui/resources/actionicons/x16/application-exit.png"))); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        jMenu1.setText(resourceMap.getString("jMenu1.text")); // NOI18N
        jMenu1.setName("jMenu1"); // NOI18N

        mnuViewWorkQueue.setAction(actionMap.get("viewWorkQueue")); // NOI18N
        mnuViewWorkQueue.setText(resourceMap.getString("mnuViewWorkQueue.text")); // NOI18N
        mnuViewWorkQueue.setName("mnuViewWorkQueue"); // NOI18N
        jMenu1.add(mnuViewWorkQueue);

        mnuViewResultsWindow.setAction(actionMap.get("viewResultsWindow")); // NOI18N
        mnuViewResultsWindow.setText(resourceMap.getString("mnuViewResultsWindow.text")); // NOI18N
        mnuViewResultsWindow.setName("mnuViewResultsWindow"); // NOI18N
        jMenu1.add(mnuViewResultsWindow);

        menuBar.add(jMenu1);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 572, Short.MAX_VALUE)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(statusMessageLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 402, Short.MAX_VALUE)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(statusAnimationLabel)
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(statusMessageLabel)
                    .addComponent(statusAnimationLabel)
                    .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(3, 3, 3))
        );

        jToolBar1.setRollover(true);
        jToolBar1.setName("toolbar"); // NOI18N

        jToggleButton3.setAction(actionMap.get("DisableAllTools")); // NOI18N
        buttonGroup1.add(jToggleButton3);
        jToggleButton3.setSelected(true);
        jToggleButton3.setText(resourceMap.getString("jToggleButton3.text")); // NOI18N
        jToggleButton3.setFocusable(false);
        jToggleButton3.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jToggleButton3.setName("jToggleButton3"); // NOI18N
        jToggleButton3.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jToggleButton3);

        jToggleButton4.setAction(actionMap.get("doCalibrate")); // NOI18N
        buttonGroup1.add(jToggleButton4);
        jToggleButton4.setIcon(resourceMap.getIcon("jToggleButton4.icon")); // NOI18N
        jToggleButton4.setText(resourceMap.getString("jToggleButton4.text")); // NOI18N
        jToggleButton4.setFocusable(false);
        jToggleButton4.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jToggleButton4.setName("jToggleButton4"); // NOI18N
        jToggleButton4.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jToggleButton4);

        jToggleButton1.setAction(actionMap.get("EnableDrawSelection")); // NOI18N
        buttonGroup1.add(jToggleButton1);
        jToggleButton1.setIcon(resourceMap.getIcon("jToggleButton1.icon")); // NOI18N
        jToggleButton1.setText(resourceMap.getString("jToggleButton1.text")); // NOI18N
        jToggleButton1.setFocusable(false);
        jToggleButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jToggleButton1.setName("jToggleButton1"); // NOI18N
        jToggleButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jToggleButton1);

        jToggleButton2.setAction(actionMap.get("EnableFillSelection")); // NOI18N
        buttonGroup1.add(jToggleButton2);
        jToggleButton2.setIcon(resourceMap.getIcon("jToggleButton2.icon")); // NOI18N
        jToggleButton2.setText(resourceMap.getString("jToggleButton2.text")); // NOI18N
        jToggleButton2.setFocusable(false);
        jToggleButton2.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jToggleButton2.setName("jToggleButton2"); // NOI18N
        jToggleButton2.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jToggleButton2);

        jSeparator2.setName("jSeparator2"); // NOI18N
        jToolBar1.add(jSeparator2);

        jButton1.setAction(actionMap.get("clearSelection")); // NOI18N
        jButton1.setIcon(resourceMap.getIcon("jButton1.icon")); // NOI18N
        jButton1.setText(resourceMap.getString("jButton1.text")); // NOI18N
        jButton1.setFocusable(false);
        jButton1.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton1.setName("jButton1"); // NOI18N
        jButton1.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jButton1);

        jSeparator3.setName("jSeparator3"); // NOI18N
        jToolBar1.add(jSeparator3);

        jButton4.setAction(actionMap.get("calcDone")); // NOI18N
        jButton4.setIcon(resourceMap.getIcon("jButton4.icon")); // NOI18N
        jButton4.setText(resourceMap.getString("jButton4.text")); // NOI18N
        jButton4.setFocusable(false);
        jButton4.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton4.setName("jButton4"); // NOI18N
        jButton4.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jButton4);

        jButton5.setAction(actionMap.get("doNextImage")); // NOI18N
        jButton5.setIcon(resourceMap.getIcon("jButton5.icon")); // NOI18N
        jButton5.setText(resourceMap.getString("jButton5.text")); // NOI18N
        jButton5.setFocusable(false);
        jButton5.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jButton5.setName("jButton5"); // NOI18N
        jButton5.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        jToolBar1.add(jButton5);

        jSeparator4.setName("jSeparator4"); // NOI18N

        jSeparator5.setName("jSeparator5"); // NOI18N

        jSeparator6.setName("jSeparator6"); // NOI18N

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
        setToolBar(jToolBar1);
    }// </editor-fold>//GEN-END:initComponents

    // <editor-fold defaultstate="collapsed" desc="Mode Select button events">
    @Action
    public void EnableDrawSelection() {
        imageCalcPanel1.setMode(ImageCalcPanel.MODE_DRAW);
    }

    @Action
    public void EnableFillSelection() {
        imageCalcPanel1.setMode(ImageCalcPanel.MODE_FILL);
    }

    @Action
    public void DisableAllTools() {
        imageCalcPanel1.setMode(ImageCalcPanel.MODE_NOOP);
    }

    @Action
    public void clearSelection() {
        imageCalcPanel1.clearSelection();
    }

    @Action
    public void doCalibrate() {
        imageCalcPanel1.setMode(ImageCalcPanel.MODE_CALIBRATE);
    }
    // </editor-fold>    
    
    // <editor-fold defaultstate="collapsed" desc="StatusMonitor code">
    public void ModeChanging(int newMode) {
        // Update the mode buttons
        jToggleButton1.setSelected(newMode == ImageCalcPanel.MODE_DRAW);
        jToggleButton2.setSelected(newMode == ImageCalcPanel.MODE_FILL);
        jToggleButton3.setSelected(newMode == ImageCalcPanel.MODE_NOOP);
        jToggleButton4.setSelected(newMode == ImageCalcPanel.MODE_CALIBRATE);
    }
    public void busy(boolean nowBusy) {
        //progressBar.setIndeterminate(nowBusy);
        if (nowBusy)
            busyIconTimer.start();
        else
            busyIconTimer.stop();
    }
    public void progressChange(int newProgress) {
        progressBar.setValue(newProgress);
    }
    public void statusMessage(String message) {
       statusMessageLabel.setText(message);
       System.out.println(message);
    }
    // </editor-fold>

    // <editor-fold defaultstate="collapsed" desc="show window code">
    /** Shows the About dialog.
     */
    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = Iasel.getApplication().getMainFrame();
            aboutBox = new IaselAboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        Iasel.getApplication().show(aboutBox);
    }    
    
    /** Shows or hides the results window depending on the view->Results Window
     * menu items checked status.
     */
    @Action
    public void viewResultsWindow() {
        rw.setVisible(mnuViewResultsWindow.isSelected());
    }    
    
    /** Shows or hides the work queue window depending on the view->Work Queue
     * menu items checked status.
     */
    @Action
    public void viewWorkQueue() {
        wq.setVisible(mnuViewWorkQueue.isSelected());
    }
    // </editor-fold>
    
    // <editor-fold defaultstate="collapsed" desc="File Menu actions">
    /** Opens an image file.
     */
    @Action
    public void fileOpenImage() {
        JFileChooser jfc = new JFileChooser();
        
        FileNameExtensionFilter filter = new 
                FileNameExtensionFilter("All Supported Image Files", 
                ImageIO.getReaderFileSuffixes());
        
        jfc.setFileFilter(filter);
        
        int returnVal = jfc.showOpenDialog(jfc);
       
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            /* Load the file */
            File file = jfc.getSelectedFile();
            
            //currentFile = file;
            //currentDone = false;
                       
            
            //imageCalcPanel1.startProcess(file);
            openImage(file);
        }
        else {
            // The user aborted.
        }
    }    
    
    @Action
    public void addFileToQueue() {
        wq.addFile();
    }

    @Action
    public void addDirectoryToQueue() {
        wq.addDirectory();
    }

    @Action
    public void exportToCSV() {
        rw.exportCSV();
    }
    // </editor-fold>   
    
    /** Completes the calculation by sending the results off to the results
     * window.
     */
    @Action
    public void calcDone() {
        if (currentFile == null) return;
        
        // If the user hasnt calibrated, ask if they want to continue.
        if (!imageCalcPanel1.calibrateDone) {
            int response = JOptionPane.showConfirmDialog(null, 
                    "You have not calibrated for this image. Are you sure " +
                    "you wish to proceed without selecting the item " +
                    "boundary? \nIf you proceed, calculations will likely " +
                    "be invalid.", "Proceed without calibration?",  
                    JOptionPane.OK_CANCEL_OPTION);
            
            if (response == JOptionPane.CANCEL_OPTION ||
                response == JOptionPane.CLOSED_OPTION)
                    return;          
        }

        imageCalcPanel1.setMode(ImageCalcPanel.MODE_NOOP);
        ModeChanging(ImageCalcPanel.MODE_NOOP);
        double percentage = imageCalcPanel1.getPercentage();
        int sPx = imageCalcPanel1.countSelectionPixels();
        int fPx = imageCalcPanel1.countFramePixels();
        
        /*String sMsg = "Frame Pixels: " + Integer.toString(fPx);
        sMsg += "                      Selected Pixels: " + Integer.toString(sPx);
        sMsg += "                      Selection Percentage is : " + Double.toString(percentage);
        imageCalcPanel1.showMessage("Results", sMsg);*/
        
        // Create the new Results Table Entry with current known values
        REntry e = new REntry();
        e.Filename = currentFile.getAbsolutePath();
        e.FramePixels = fPx;
        e.SelectedPixels = sPx;
        e.SelectedPercentage = percentage;
        
        // Prompt the user for the images tag
        e.Tag = JOptionPane.showInputDialog("Enter tag for this image:");
        
        // Perform area calculations if enabled.
        if (globals.AreaCalcEnable) {
            // Grab the list of all area profiles
            ArrayList<AreaProfile> apl = globals.areaProfiles.getProfiles();
            
            // The area profile to use will live in here.
            AreaProfile ap = null;
            
            // If there are no profiles, there is nothing to do.
            if (!apl.isEmpty()) {
                if (apl.size() == 1) {
                    // There is only one area profile present. No point asking
                    // the user to pick one out of the one profiles available.
                    System.out.println("Only one area profile - using that.");
                    ap = apl.get(0);
                }
                else {
                    // There are multiple area profiles. We will have to ask the
                    // user which one to use.
                    Object response = JOptionPane.showInputDialog(null, 
                            "Multiple Area Profiles are available. Which one " +
                            "applies to this sample?", "Select Area Profile", 
                            JOptionPane.QUESTION_MESSAGE, null, apl.toArray(), 
                            apl.get(0));
                    if (response == null)
                        ap = null;
                    else
                        ap = (AreaProfile)response;
                    
                    /* This code does the same as above except with a different
                     * dialog.
                    int response = JOptionPane.showOptionDialog(null, 
                            "Multiple Area Profiles are available. Which one " +
                            "applies to this sample?", "Select Area Profile", 
                            JOptionPane.OK_CANCEL_OPTION, 
                            JOptionPane.QUESTION_MESSAGE, null, apl.toArray(), 
                            apl.get(0));
                    if (response != JOptionPane.CLOSED_OPTION) {
                        ap = apl.get(response);
                        System.out.println("User Chose Area Profile: " + ap.display_name);
                    }
                     */
                }
            }
            else {
                System.out.println("No area profiles present in file");
            }
            
            if (ap != null) {
                // An area profile has been selected. Perform the calculations!
                e.AreaType = ap.short_name;
                
                double totalArea = ap.width * ap.depth;
                
                double apc = e.SelectedPercentage / 100;
                
                e.SelectedArea = apc * totalArea;
            }
        }
        
        
        rw.AddEntry(e);
        currentDone = true;
        
        if (fromQueue) {
            wq.finishFile();
            fromQueue = false;
        }
        
        // Reset the calc panel
        imageCalcPanel1.reset();
    }

    /** Moves on to the next image, marking the current one as Done if it hasnt
     * already.
     */
    @Action
    public void doNextImage() {
        // Complete the current file if it hasnt been completed already
        if (!currentDone) calcDone();
        
        // Return if there is nothing more in the queue
        if (wq.isEmpty()) return;
        
        // Ensure the next one isnt beyond the end of the queue
        if (wq.getRemaining() == 0) return;
        
        // Grab the next item from the queue then get to work on it.
        String fn = wq.workOnNext();
        if (fn == null) {
            System.err.println("ERROR: List is not empty but no file was returned");
            return;
        }
        if (fn.isEmpty()) {
            System.err.println("ERROR: empty filename was returned");
            return;
        }
        
        fromQueue = true;
        openImage(fn);
    }
    
    /** Opens the file specified by the given filename and starts the process
     * running on it.
     * 
     * @param filename The file to work on.
     */
    public void openImage(String filename) {
        File f = new File(filename);
        openImage(f);
    }    
    
    /** Opens the specified image file and starts the process running.
     * @param file The file to work on.
     */
    public void openImage(File file) {
        currentFile = file;
        currentDone = false;
        imageCalcPanel1.reset();
        //imageCalcPanel1.setMonitor(this);
        imageCalcPanel1.startProcess(file);
    }
    

    
    // <editor-fold defaultstate="collapsed" desc="Form-delcared variables">    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup1;
    private iasel.gui.ImageCalcPanel imageCalcPanel1;
    private javax.swing.JButton jButton1;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton5;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JToolBar.Separator jSeparator2;
    private javax.swing.JToolBar.Separator jSeparator3;
    private javax.swing.JSeparator jSeparator4;
    private javax.swing.JSeparator jSeparator5;
    private javax.swing.JSeparator jSeparator6;
    private javax.swing.JSeparator jSeparator7;
    private javax.swing.JSeparator jSeparator8;
    private javax.swing.JToggleButton jToggleButton1;
    private javax.swing.JToggleButton jToggleButton2;
    private javax.swing.JToggleButton jToggleButton3;
    private javax.swing.JToggleButton jToggleButton4;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JMenuItem mnuFileAddDirectory;
    private javax.swing.JMenuItem mnuFileAddFile;
    private javax.swing.JMenuItem mnuFileExportCSV;
    private javax.swing.JMenuItem mnuFileOpen;
    private javax.swing.JCheckBoxMenuItem mnuViewResultsWindow;
    private javax.swing.JCheckBoxMenuItem mnuViewWorkQueue;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    // End of variables declaration//GEN-END:variables
    // </editor-fold>
    
    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;

    private JDialog aboutBox;
}
